# local
from epibox.common.close_file import close_file

def disconnect_system(devices, a_file=None, files_open=True, devices_connected=True): 

    if devices_connected:
        for device in devices:
            try:
                device.stop()
                device.close()
                    
            except:
                continue

    if files_open:
        close_file(a_file)







# built-in
import time
import sys
from datetime import datetime

# third-party
import numpy as np

# local 
from epibox.common.read_modules import read_modules
from epibox.common.close_file import close_file
from epibox.common.write_file import write_file
from epibox.common.open_file import open_file


def run_system(devices, a_file, sync_param, directory, mac_channels, sensors, fs, save_fmt, header):
    
    if time.time()-sync_param['strtime'] > 5:

        sync_param['strtime'] = time.time()
        now = datetime.now()
        sync_time = now.strftime("%Y-%m-%d %H:%M:%S.%f").rstrip('0')
        sync_param['sync_time'] = sync_time
        sync_param['connection'] = 1
        sync_param['sync_append'] = 0
    
    for i,device in enumerate(devices):
        sync_param['sync_arr_'+chr(ord('@')+i+1)] = np.zeros(1000, dtype = float)
    
    now = datetime.now()
    sync_param['sync_time'] = now.strftime("%Y-%m-%d %H:%M:%S.%f")
    t, t_str, t_display = read_modules(devices, mac_channels, sensors, header)

    
    # Open new file each hour
    if time.time()-sync_param['inittime'] > 60*60:
        sync_param['close_file'] = 1

    i = time.time() - sync_param['inittime']

    # print elapsed time
    sys.stdout.write("\rElapsed time (seconds): % i " % i)
    sys.stdout.flush()
    
    try:
        write_file(t, a_file, sync_param, str(i), save_fmt)
        
    except Exception as e:
        print(e)

    # Open new file each hour
    if sync_param['close_file'] == 1:
        # close the file
        print('closing')
        close_file(a_file)

        # Open a new file
        print('Opening new file')
        a_file, save_fmt = open_file(directory, devices, mac_channels, sensors, fs)

        sync_param['close_file'] = 0
        sync_param['inittime'] = time.time()

    # -----------------------------------------------------------------
    return t, t_display, a_file, sync_param



# built-in
from datetime import datetime
import os

# local
from epibox.bit.header2bitalino import header2bitalino

def open_file(directory, devices, mac_channels, sensors, fs, save_raw, service):

    # for txt format
    now = datetime.now()
    save_time = now.strftime("%Y-%m-%d %H-%M-%S").rstrip('0')
    file_time = now.strftime("%Y-%m-%d %H:%M:%S.%f").rstrip('0')
    file_time = file_time[11:]
    file_time = '"' + file_time + '"'
    file_date = '"' + save_time[0:10] + '"'
    
    a_file = open(os.path.join(directory, 'A' + save_time + '.txt'), 'w') # data file
    save_fmt, header = header2bitalino(a_file, file_time, file_date, devices, mac_channels, sensors, fs, save_raw, service)
    

    return a_file, save_fmt, header



# built-in
import json

# local
from epibox.bit.manage_devices import pause_devices, connect_devices, start_devices
from epibox.exceptions.exception_manager import error_disconnect, error_kill, client_kill
from epibox.common.setup import setup_client, setup_config, setup_variables
from epibox.common.create_folder import create_folder
from epibox.common.open_file import open_file
from epibox.common.write_file import write_annot_file
from epibox.common.run_system import run_system
from epibox.common import process_data
from epibox.bit.get_battery import get_battery


# ****************************** MAIN SCRIPT ***********************************


def main():

    devices = []

    try:
        client = setup_client()
        opt, channels, sensors, service, save_raw = setup_config(client)
        t_all, already_notified_pause, system_started, files_open = setup_variables()

        # Use/create the patient folder ===============================================================
        directory = create_folder(opt['initial_dir'], '{}'.format(opt['patient_id']), service)
        already_timed_out = False

        devices = connect_devices(client, devices, opt, already_timed_out, files_open=False)
    
    except Exception as e:
        error_kill(client, devices, msg='Failed initial setup', files_open=False, devices_connected=False)

    try:
        a_file, save_fmt, header = open_file(directory, devices, channels, sensors, opt['fs'], save_raw, service)
        files_open = True

    except Exception as e:
        error_kill(client, devices, msg='Failed to open the files', files_open=False)


    # Starting Acquisition LOOP =========================================================================
    try:
        while client.keepAlive == True:

            if client.newAnnot != None:
                print(f'annot: {client.newAnnot}')
                write_annot_file(a_file.name, client.newAnnot)
                client.newAnnot = None

            if client.pauseAcq and not already_notified_pause:

                devices = pause_devices(client, devices)
                already_notified_pause = True

            elif not client.pauseAcq:

                if already_notified_pause:
                    client.publish('rpi', str(['RECONNECTING']))
                    already_notified_pause = False

                if not system_started:

                    get_battery(client, devices, service)
                    
                    try:
                        sync_param = start_devices(client, devices, opt['fs'], channels, header)
                        system_started = True
                        already_timed_out = False

                    except Exception as e:
                        print(e)
                        pass

                # if time.time() - start_time > 5*60:
                #     client.keepAlive = False

                try:
                    _, t_disp, a_file, sync_param = run_system(devices, a_file, sync_param, directory, channels, sensors, opt['fs'], save_fmt, header)

                    t_display = process_data.decimate(t_disp, opt['fs'])
                    t_all += t_display[0]

                    json_data = json.dumps(['DATA', t_display, channels, sensors])
                    client.publish('rpi', json_data)

                    already_timed_out = False

                # Handle misconnection of the devices--------------------------------------------------------------------------------------------
                except Exception as e:
                    
                    devices, system_started = error_disconnect(client, devices, e, directory, a_file)

                    # Reconnect the devices
                    devices = connect_devices(client, devices, opt, already_timed_out, a_file)

                    system_started = False
                    a_file, save_fmt, header = open_file(directory, devices, channels, sensors, opt['fs'], save_raw, service)

            else:
                pass

        client_kill(client, devices, 'You have stopped the acquisition', a_file, files_open)

    except KeyboardInterrupt:
        client_kill(client, devices, 'You have stopped the acquisition', a_file, files_open)


    # =========================================================================================================


# built-in
import time
from datetime import datetime
import json

# third-party
import numpy as np

# local
from epibox.exceptions.exception_manager import error_kill
from epibox.common.connect_device import connect_device


def start_devices(client, devices, fs, mac_channels, header):

    dig_Out = 0
    now = datetime.now()
    sync_param = {'flag_sync' : 0 , 'inittime' : time.time(), 'strtime': time.time(), 'sync_time' : now.strftime("%Y-%m-%d %H:%M:%S.%f").rstrip('0'), 'dig_Out' : dig_Out, 'close_file' : 0, 'mode': 0, 'diff': 1000, 'save_log': 1, 'count_a': 1000, 'sync_append': 0}
    # mode: 0 if not started acquisition yet (or if paused) and 1 otherwise (used to write in drift_log_file)
    
    for i in range(len(devices)):
        sync_param['sync_arr_'+chr(ord('@')+i+1)] = np.zeros(1000, dtype = float)

    # Initialize devices
    for device in devices:

        if header['service'] == 'Bitalino' or header['service'] == 'Mini':
            channels = [int(elem[1])-1 for elem in mac_channels if elem[0]==device.macAddress]
        else:
            channels = [int(elem[1]) for elem in mac_channels if elem[0]==device.macAddress]
            
        device.start(SamplingRate=fs, analogChannels=channels)

    now = datetime.now()
    print('start {}'.format(now))
    sync_param['sync_time'] = now.strftime("%Y-%m-%d %H:%M:%S.%f")

    client.publish('rpi', str(['ACQUISITION ON']))                 
    
    return sync_param


def connect_devices(client, devices, opt, already_timed_out, a_file=None, files_open=True):

    for mac in opt['devices_mac']:

        init_connect_time = time.time()
        print('Searching for Module...' + mac)

        i = 0
        while client.keepAlive:

            i += 1

            if (time.time() - init_connect_time) > 120:
                error_kill(client, devices, 'Failed to reconnect to devices', 'ERROR', a_file, files_open)

            try:
                
                connected = False
                time.sleep(3)
                connected, devices = connect_device(mac, client, devices)

                if connected and mac in [d.macAddress for d in devices]:
                    now = datetime.now()
                    save_time = now.strftime("%H-%M-%S").rstrip('0')
                    break

                else:
                    if time.time() - init_connect_time > 3*i:
                        timeout_json = json.dumps(['TRYING TO CONNECT', '{}'.format(mac)])
                        client.publish('rpi', timeout_json)
                    raise Exception

            except Exception as e:

                if not already_timed_out and (time.time() - init_connect_time > 3*i):

                    timeout_json = json.dumps(['TIMEOUT', '{}'.format(mac)])
                    client.publish('rpi', timeout_json)

                    already_timed_out = True

                continue

    return devices



def pause_devices(client, devices):

    for device in devices:

        try:
            device.stop()
        except Exception as e:
            print(e)
            continue

    client.publish('rpi', str(['PAUSED']))

    return devices




# built-in
from http import client
import pwd
import os
import ast

# third-party
import paho.mqtt.client as mqtt

# local
from epibox.mqtt_manager.message_handler import on_message, send_default
from epibox.mqtt_manager.utils import random_str

def setup_client():
    client_name = random_str(6)
    print('Client name (acquisition):', client_name)
    host_name = '192.168.0.10'
    topic = 'rpi'

    client = mqtt.Client(client_name)

    setattr(client, 'keepAlive', True)
    setattr(client, 'pauseAcq', False)
    setattr(client, 'newAnnot', None)

    client.username_pw_set(username='preepiseizures', password='preepiseizures')
    client.connect(host_name)
    client.subscribe(topic)
    client.on_message = on_message
    client.loop_start()
    print('Successfully subcribed to topic', topic)

    client.publish('rpi', str(['STARTING']))

    return client


def setup_config(client):
    
    username = pwd.getpwuid(os.getuid())[0]

    send_default(client, username)

    with open('/home/{}/Documents/epibox/args.json'.format(username), 'r') as json_file:
        opt = json_file.read()
        opt = ast.literal_eval(opt)

    try: 
        opt['devices_mac'].values()
    except Exception as e:
        opt['devices_mac'] = {'MAC1': '12:34:56:78:91:10','MAC2': ''}

    if not opt['channels']:
        channels = []
        for device in opt['devices_mac'].values():
            for i in range(1, 7):
                channels += [[device, str(i)]]
        sensors = ['-' for i in range(len(channels))]

    else:
        channels = []
        sensors = []
        try:
            for triplet in opt['channels']:
                triplet[0] = opt['devices_mac'][triplet[0]] # replace MAC ID for corresponding MAC
                channels += [triplet[:2]]
                sensors += [triplet[2]]
        except Exception as e:
            print(e)
            for tt,triplet in enumerate(opt['channels']):
                if tt < 7:
                    triplet[0] = opt['devices_mac']['MAC1'] 
                else: 
                    triplet[0] = opt['devices_mac']['MAC2']
                channels += [triplet[:2]]
                sensors += [triplet[2]]


    if 'save_raw' in opt.keys(): save_raw = bool(opt['save_raw'])
    else: save_raw = bool(opt['saveRaw'])

    service = opt['service']
    opt['devices_mac'] = [m for m in opt['devices_mac'].values() if m != '']

    if opt['initial_dir'] == 'EpiBOX Core':
        opt['initial_dir'] = '/home/{}/Documents/epibox/acquisitions'.format(username)
    else: 
        if os.path.isdir('/media/{}/'.format(username) + opt['initial_dir']):
            opt['initial_dir'] = '/media/{}/'.format(username) + opt['initial_dir'] + '/acquisitions'
        else:
            opt['initial_dir'] = '/home/{}/Documents/epibox/acquisitions'.format(username)

    print('ID: {}'.format(opt['patient_id']))
    print('folder: {}'.format(opt['initial_dir']))
    print('fs: {}'.format(opt['fs']))
    print('save_raw: {}'.format(save_raw))
    print('channels: {}'.format(channels))
    print('devices: {}'.format(opt['devices_mac']))
    print('sensors: {}'.format(sensors))
    print('service: {}'.format(service))

    return opt, channels, sensors, service, save_raw


def setup_variables():

    already_notified_pause = False
    system_started = False
    files_open = False 
    t_all = []

    return t_all, already_notified_pause, system_started, files_open




import numpy as np

def detect_event(t, sensors):
    strength_list = [0] * len(sensors)

    for i,chn in enumerate(t):
        
        if sensors[i] == 'EMG':
            print(f'sum square: {np.sum(np.square(chn-np.mean(chn)))} | variance: {np.var(chn)} | mean sqrt: {np.mean(np.sqrt(chn))}\n')
            if np.sum(np.sum(np.square(chn-np.mean(chn)))) > 20000:
                strength_list[i] = 1
            if np.sum(np.sum(np.square(chn-np.mean(chn)))) > 60000:
                strength_list[i] = 2


    return strength_list





#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()



test_requirements = [ ]

setup(
    author="Ana Sofia Carmo",
    author_email='anascacais@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="EpiBOX is a Raspberry Pi tool for easy signal acquisition.",
    entry_points={
        'console_scripts': [
            'epibox=epibox.cli:main',
        ],
    },
    install_requires=['alabaster==0.7.12', 'appdirs==1.4.4', 'argh==0.26.2', 'arrow==1.1.1', 'Babel==2.9.1', 'binaryornot==0.4.4', 'bitalino==1.2.1', 'bleach==3.3.0', 'certifi==2021.5.30', 'chardet==4.0.0', 'click', 'colorama==0.4.4', 'cookiecutter==1.7.3', 'distlib==0.3.2', 'docutils==0.17.1', 'entrypoints==0.3', 'filelock==3.0.12', 'idna==2.10', 'imagesize==1.2.0', 'install==1.3.4', 'Jinja2==3.0.1', 'jinja2-time==0.2.0', 'MarkupSafe==2.0.1', 'mccabe==0.6.1', 'numpy==1.21.0', 'packaging==21.0', 'paho-mqtt==1.5.1', 'pathtools==0.1.2', 'pexpect==4.8.0', 'pkginfo==1.7.0', 'pluggy', 'poyo==0.5.0', 'primefac==1.1', 'ptyprocess==0.7.0', 'py==1.10.0', 'pybluez2==0.41', 'pycodestyle==2.5.0', 'pyflakes==2.1.1', 'Pygments==2.9.0', 'pyparsing==2.4.7', 'pyserial==3.5', 'python-dateutil==2.8.1', 'python-slugify==5.0.2', 'pytz==2021.1', 'PyYAML==5.4.1', 'readme-renderer==29.0', 'requests==2.25.1', 'requests-toolbelt==0.9.1', 'scipy==1.7.0', 'six==1.16.0', 'snowballstemmer==2.1.0', 'Sphinx==1.8.5', 'sphinxcontrib-serializinghtml==1.1.5', 'sphinxcontrib-websupport==1.2.4', 'text-unidecode==1.3', 'toml==0.10.2', 'tox', 'tqdm==4.61.2', 'twine', 'urllib3==1.26.6', 'virtualenv==20.4.7', 'watchdog==0.9.0', 'webencodings==0.5.1'],
    license="MIT license",
    long_description_content_type='text/markdown',
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords=['epibox', 'signal acquisition', 'Raspberry Pi'],
    name='epibox',
    packages=find_packages(include=['epibox', 'epibox.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/anascacais/epibox',
    version='1.1.0',
    zip_safe=False,
)
